@@ -1,6 +1,6 @@
 import networkx as nx
 import matplotlib.pyplot as plt
-import io as StringIO
+from io import StringIO
 import tkinter as tk
 
 from contextlib import redirect_stdout
@@ -84,6 +84,17 @@ class Cell():
     def get_copy(self):
         return Cell(self.name, self.content_type, self.content, self.stdout)
 
+    """
+    @contextlib.contextmanager
+    def stdoutIO(self, stdout=None):
+        old = sys.stdout
+        if stdout is None:
+            stdout = StringIO.StringIO()
+        sys.stdout = stdout
+        yield stdout
+        sys.stdout = old
+    """
+
     def execute(self):
         # Execute this cell's content
 
@@ -101,10 +112,8 @@ class Cell():
                 print(e)
         elif self.stdout == "external":
             try:
-                f = StringIO()
-                with redirect_stdout(f):
-                    exec(self.content, ex_vars_copy)
-                self.output = f.getvalue()
+                print("<" + self.name + ">")
+                exec(self.content, ex_vars_copy)
             except Exception as e:
                 print("Exception occurred in cell " + self.name)
                 print(e)
@@ -119,9 +128,10 @@ class Cell():
 
 class Graph:
 
-    def __init__(self):
+    def __init__(self, parent):
         # Networkx Directed graph
         self.graph = nx.DiGraph()
+        self.parent = parent
         # Dict to keep track of cell names vs networkx node names
         self.names_to_indeces = {}
         # Dictionaries for variables created by cells
@@ -130,6 +140,8 @@ class Graph:
         # TextIO object
         self.ti = TextIO()
 
+        self.executing = False
+
     def get_lookup_table(self):
         return {idx: moniker for moniker, idx in
                 zip(list(self.names_to_indeces.keys()), list(self.names_to_indeces.values()))}
@@ -143,6 +155,7 @@ class Graph:
         :param cell_name: Name of cell
         :return: Corresponding index of provided cell name
         """
+        cell_name = cell_name.strip()
         if cell_name not in self.get_all_cells_edges()[0]:
             print("Cell \"" + cell_name + "\" does not exist")
             return -1
@@ -170,6 +183,7 @@ class Graph:
         :param new_cell: Cell object to be added to graph
         """
         if new_cell.name in list(self.names_to_indeces.keys()):
+            print(new_cell.name)
             print("All cells must have unique names")
             return
         else:
@@ -194,6 +208,7 @@ class Graph:
             del self.names_to_indeces[cell_name]
         else:
             print("Cell \"" + cell_name + "\" does not exist.")
+            return -1
 
     def connect_cells(self, idx1, idx2):
         """
@@ -328,8 +343,14 @@ class Graph:
             with open(output_filename, 'w') as txt:
                 txt.write(std_file_out)
 
-    def bfs_traversal_execute(self, stdout="internal", output_filename="stdout.txt"):
-        std_file_out = ""
+    def bfs_traversal_execute(self, stdout="external", output_filename="stdout.txt"):
+        import time
+        if len(self.get_all_cells_edges()[0]) == 0:
+            return
+
+        std_file_out = "<root>"
+
+        self.executing = True
 
         root_cell = self.get_cell("", 0)
         root_cell.stdout = stdout
@@ -339,15 +360,18 @@ class Graph:
         root.start()
         root.join()
 
+        time.sleep(.05)
+
         std_file_out += root_cell.output
 
-        neighbors = self.graph.neighbors(0)
+        n_ = self.graph.neighbors(0)
+        neighbors = [n for n in n_]
 
         while neighbors:
             new_neighbors = []
             processes = []
             for n in neighbors:
-                neighbor_cell = self.get_cell(self.get_lookup_table()[n])
+                neighbor_cell = self.get_cell(self.get_lookup_table()[n].strip())
                 neighbor_cell.stdout = stdout
 
                 neighbor = threading.Thread(target=neighbor_cell.execute)
@@ -355,19 +379,19 @@ class Graph:
                 neighbor.start()
                 processes.append(neighbor)
 
-                new_neighbors.extend(self.graph.neighbors(n))
+                new_neighbors.extend([i for i in self.graph.neighbors(n)])
 
             [proc.join() for proc in processes]
 
             for n in neighbors:
+                time.sleep(.05)
                 neighbor = self.get_cell(self.get_lookup_table()[n])
+                std_file_out += "<" + neighbor.name + ">\n"
                 std_file_out += neighbor.output
 
             neighbors = new_neighbors
 
-        if stdout == "external":
-            with open(output_filename, 'w') as txt:
-                txt.write(std_file_out)
+        self.executing = False
 
     def save_graph(self, filename):
         txtout = ""
@@ -385,7 +409,7 @@ class Graph:
                 fill_with_code = "n\n"
             temp_text = "cell " + c.name + " " + c.content_type + " " + fill_with_code
             if fill_with_code == "y:\n":
-                temp_text += c.content + ";\n"
+                temp_text += c.content + "\n;\n"
 
             txtout += temp_text
 
@@ -397,20 +421,70 @@ class Graph:
         with open(filename, "w+") as file:
             file.write(txtout)
 
+    def get_satx_as_txt(self):
+        txtout = ""
+
+        lookup_table = self.get_lookup_table()
+        cell_names, edges, _ = self.get_all_cells_edges()
+        cells = [self.get_cell(cn) for cn in cell_names]
+
+        for c in cells:
+            if c.content:
+                fill_with_code = "y:\n"
+            else:
+                fill_with_code = "n\n"
+            temp_text = "cell " + c.name + " " + c.content_type + " " + fill_with_code
+            if fill_with_code == "y:\n":
+                temp_text += c.content + "\n;\n"
+
+            txtout += temp_text
+
+        for e in edges:
+            name1 = lookup_table[e[0]]
+            name2 = lookup_table[e[1]]
+            txtout += "link " + name1 + " " + name2 + "\n"
+
+        if self.parent.std_capture.getvalue():
+            txtout += "<!--SATYRN_DCO_START-->\n"
+            txtout += self.parent.std_capture.getvalue()
+            txtout += "<execution complete>\n<!--SATYRN_DCO_END-->"
+
+        return txtout
+
+    def get_py_file(self):
+        txtout = ""
+
+        lookup_table = self.get_lookup_table()
+        cell_names, edges, _ = self.get_all_cells_edges()
+        cells = [self.get_cell(cn) for cn in cell_names]
+
+        for c in cells:
+            txtout += "# <" + c.name + ">\n"
+            if c.content_type == "python":
+                txtout += c.content + "\n"
+            else:
+                txtout += "\"\"\"\n" + c.content + "\n\"\"\"\n"
+
+        txtout += "# <EOF>"
+
+        return txtout
+
 
 class Interpreter:
 
     def __init__(self):
         # Graph object
-        self.graph = Graph()
+        self.graph = Graph(self)
         # Assume live input first
         self.input_type = "live"
         # This will be set if the user executes a .satx file
         self.file = None
         # This determines whether or not stdout gets sent to an external textbox
-        self.stdout = "internal"
+        self.stdout = "external"
         self.stdout_filename = "stdout.txt"
         # Start loop
+        self.std_capture = StringIO()
+
         self.run()
 
     def run_file(self, command):
@@ -420,10 +494,90 @@ class Interpreter:
         try:
             openfile = open(command[0], "r")
             self.file = openfile.readlines()
-            self.input_type = "file"
+            self.run_string(self.file)
         except Exception as e:
             print(e)
 
+    def run_string(self, content):
+        self.file = content
+        self.input_type = "file"
+
+        reading_dco_output = False
+
+        while len(content) > 0:
+            command = content.pop(0).split(" ")
+
+            if "<!--SATYRN_DCO_START-->" in command[0]:
+                reading_dco_output = True
+                continue
+            if "<!--SATYRN_DCO_END-->" in command[0]:
+                reading_dco_output = False
+                continue
+
+            if reading_dco_output:
+                self.std_capture.write(" ".join(command) + "\n")
+                continue
+
+            if len(command) == 0 or command == '':
+                continue
+
+            elif command[0] == "help":
+                print(self.help_menu())
+
+            elif command[0] == "quit":
+                break
+
+            elif command[0] == "cell":
+                self.create_cell(command)
+
+            elif command[0] == "edit":
+                self.edit_cell(command)
+
+            elif command[0] == "rename":
+                self.rename_cell(command)
+
+            elif command[0] == "remove":
+                self.remove_cell(command)
+
+            elif command[0] == "link":
+                self.link(command)
+
+            elif command[0] == "sever":
+                self.sever(command)
+
+            elif command[0] == "merge":
+                self.merge(command)
+
+            elif command[0] == "swap":
+                self.swap(command)
+
+            elif command[0] == "execute":
+                self.execute(command)
+
+            elif command[0] == "display":
+                self.display(command)
+
+            elif command[0] == "list":
+                self.list_cells()
+
+            elif command[0] == "stdout":
+                self.set_stdout(command)
+
+            elif command[0] == "reset_runtime":
+                self.reset_runtime()
+
+            elif command[0] == "reset_graph":
+                self.reset_graph()
+
+            elif command[0] == "save":
+                self.save_graph(command)
+
+            elif ".satx" in command[0]:
+                self.run_file(command)
+
+            elif command[0]:
+                print("Syntax error: command \"" + command[0] + "\" not recognized.")
+
     def read_input(self):
         # Read input from stdin or external file. Returns list of command params.
         if self.input_type == "live" or len(self.file) == 0:
@@ -454,12 +608,12 @@ class Interpreter:
             "display [cell_name]": "Displays graph. If cell_name defined, that cell's details will be printed out",
             "list": "Prints out names of all cells in graph",
             "reset_runtime": "Deletes all variables created within cells",
-            "reset_graph": "Deletes all variables and cells. Equivalent to restarting satyrnCLI session",
+            "reset_graph": "Deletes all variables and cells. Equivalent to restarting satyrn_python session",
             "save [filename].satx": "Saves graph to .satx file",
-            "[filename].satx": "Executes satyrnCLI code in specified file. File must have .satx extension. "
+            "[filename].satx": "Executes satyrn_python code in specified file. File must have .satx extension. "
                                "\n\t\tExamples of "
                                "syntax can be seen at https://github.com/CharlesAverill/satyrn/tree/master/examples ",
-            "quit": "Exits satyrnCLI session"
+            "quit": "Exits satyrn_python session"
         }
         output = ("------------------------------------------------------------------------\n"
                   "Hi, and welcome to Satyrn.\n"
@@ -514,7 +668,7 @@ class Interpreter:
         :param command: command to be executed
         """
         if len(command) != 2:
-            print("link takes 1 arguments: [cell_name]")
+            print("edit takes 1 argument: [cell_name]")
             return
 
         target_cell = self.graph.get_cell(command[1])
@@ -525,6 +679,10 @@ class Interpreter:
 
         target_cell.content = new_content
 
+    def set_cell_contents(self, command):
+        target_cell = self.graph.get_cell(command[1])
+        target_cell.content = command[2]
+
     def rename_cell(self, command):
         """
         :param command: command to be executed
@@ -551,7 +709,9 @@ class Interpreter:
         """
         to_remove = command[1:]
         for cell in to_remove:
-            self.graph.remove_cell(cell)
+            i = self.graph.remove_cell(cell)
+            if i == -1:
+                return -1
 
     def link(self, command):
         """
@@ -563,14 +723,15 @@ class Interpreter:
 
         idx1 = self.graph.name_to_idx(command[1])
         idx2 = self.graph.name_to_idx(command[2])
-
+        """
         if idx2 == 0:
             confirm = input("WARNING: You are attempting to connect a node to your root node. This could cause unwanted"
                             " recursive behavior. Are you sure? (y/n) ")
             if "y" in confirm.lower():
                 self.graph.connect_cells(idx1, idx2)
         else:
-            self.graph.connect_cells(idx1, idx2)
+        """
+        self.graph.connect_cells(idx1, idx2)
 
     def sever(self, command):
         """
@@ -678,11 +839,18 @@ class Interpreter:
         global exec_vars
         exec_vars = {}
 
-    def reset_graph(self):
-        confirm = input("Are you sure you want to reset the graph? This will delete all nodes and variables. (y/n) ")
-        if "y" in confirm:
-            self.graph = Graph()
+    def reset_graph(self, ask=True):
+        if (ask):
+            confirm = input(
+                "Are you sure you want to reset the graph? This will delete all nodes and variables. (y/n) ")
+            if "y" in confirm:
+                self.graph = Graph(self)
+                self.reset_runtime()
+                self.std_capture = StringIO()
+        else:
+            self.graph = Graph(self)
             self.reset_runtime()
+            self.std_capture = StringIO()
 
     def save_graph(self, command):
         """
