@@ -5,7 +5,7 @@ import webbrowser
 
 from cheroot.wsgi import Server as WSGIServer, PathInfoDispatcher
 
-from satyrn_flask import create_app
+from .satyrn_flask import create_app
 
 
 def delayed_browser_open():
