@@ -88,20 +88,21 @@ $("#file-input").change(function(e){
                     }
 
                     $("#file-input").val(null);
-                    /*
+
                     $.ajax({
                         type : "GET",
                         url : "/dynamic_cell_output/",
                         success: function (data) {
                             if(data.includes("<!--SATYRN_DONE_EXECUTING-->")){
-                                is_executing = false;
-                                just_finished = true;
                                 data = data.substring(28);
                             }
+                            if(data.includes("<execution complete>")){
+                                data = data.substring(0, data.lastIndexOf("<execution complete>"));
+                            }
                             updateDCO(data);
                         }
                     });
-                    */
+
                 }
             }
         });
